# urls
AUTH_URL = "https://weblogin.umich.edu/cosign-bin/cosign.cgi"
AUTH_PAGE_URL = "https://weblogin.umich.edu"
API_BASE_URL = "https://leccap.engin.umich.edu/leccap/viewer/api/recordings/"

DEFAULT_CONCURRENCY = 5
DEFAULT_MSG = ""
DEFAULT_DIR = "."


from builtins import object
import os
import json

"""
Simple JSON based config manager

@Version: 2.0
"""
class ConfigParser(object):

    def __init__(self):
        """
        Read config dict from file
        """
        self._json_path = os.path.join(os.path.dirname(__file__), 'config.json')
        with open(self._json_path) as f:
            self._config = json.load(f)
    
    def get(self, key):
        """
        Get a value from the json config using dot accessor
        
        Arguments:
            key {str} -- Key
        
        Returns:
            mixed -- value for the key
        """
        return self._get(self._config, key.split('.'))
    
    def set(self, key, value):
        """
        Set a value to the config
        
        Arguments:
            key {str} -- Key
            value {str|number|list|dict} -- Value
        """
        self._set(self._config, key.split('.'), value)
    
    def save(self):
        """
        Dump back to the file
        """
        with open(self._json_path, 'w') as f:
            json.dump(self._config, f)

    """
    Helpers
    """
    def _get(self, config, keys):
        curr_key = keys[0]
        if len(keys) == 1:
            return config[curr_key]
        if not self._is_list(config[curr_key]) and not self._is_dict(config[curr_key]):
            raise ValueError("Cannot access object that is not list nor dict!")
        return self._get(config[curr_key], keys[1:])
    
    def _set(self, config, keys, value):
        curr_key = keys[0]
        if len(keys) == 1:
            config[curr_key] = value
        else:
            if curr_key not in config:
                raise ValueError("Key not exists!")
            self._set(config[curr_key], keys[1:], value)
    
    def _is_list(self, obj):
        return isinstance(obj, list)
    
    def _is_dict(self, obj):
        return isinstance(obj, dict)



from __future__ import print_function
from builtins import range
from colorama import init, Fore, Style

"""
Utility functions

@Version 2.0.0
"""

def init_terminal():
    """
    Initialize terminal color settings
    """
    init()

def print_info(text, ret=False):
    s = Fore.CYAN + text
    s += Style.RESET_ALL
    if ret:
        return s
    else:
        print(s)

def print_success(text, ret=False):
    s = Fore.GREEN + text
    s += Style.RESET_ALL
    if ret:
        return s
    else:
        print(s)

def print_error(text, ret=False):
    s = Fore.RED + text
    s += Style.RESET_ALL
    if ret:
        return s
    else:
        print(s)

def print_warning(text, ret=False):
    s = Fore.YELLOW + text
    s += Style.RESET_ALL
    if ret:
        return s
    else:
        print(s)

def chunks(l, n):
    """
    Slice list into chunks
    
    Arguments:
        l {list}
        n {int} -- Num items per chunk
    """

    # For item i in a range that is a length of l,
    for i in range(0, len(l), n):
        # Create an index range for l of n items:
        yield l[i:i+n]



#!/usr/bin/python
from __future__ import absolute_import
import os
import sys
from .utils import *
from .constants import *
from .auth import Authenticator
from .config import ConfigParser
from .download import Downloader

"""
University Of Michigan Lecture Videos Downloader 

@Authored By: Maxim Aleksa maximal@umich.edu
@Updated By: Jackie Zhang jackierw@umich.edu
@Version: 2.0.0

Usage: 
leccap dl url
leccap reset [logins|path|concurrency|all]
leccap config [login.username|login.password|concurrency] [$value]
"""

def download(config, url):
    # extract dest path
    dest_path = config.get('dest_path')
    if dest_path == '.':
        dest_path = os.getcwd()
    downloader = Downloader(url, dest_path)
    # set concurrency``
    concurrency = config.get('concurrency')
    downloader.set_concurrency(concurrency)
    # check authentication
    if downloader.requires_auth():
        auth = Authenticator()
        # check for saved credentials
        username = config.get('logins.username')
        password = config.get('logins.password')
        if username and password:
            print_info("Using saved credentials...")
            auth.ask_for_credentials(username=username, password=password)
        else:
            print_warning("Needs authentication. But you can save your credentials using ./leccap config! ")
            auth.ask_for_credentials()
        downloader.set_auth(auth)
        if not downloader.get_auth().is_authenticated():
            print_info("Authenticating...")
            downloader.get_auth().authenticate()
    # start download
    downloader.start()

def reset_config(config, key):
    if key == 'all':
        # reset all
        config.set('logins', {'username': DEFAULT_MSG, 'password': DEFAULT_MSG})
        config.set('concurrency', DEFAULT_CONCURRENCY)
        config.set('dest_path', DEFAULT_DIR)
    else:
        # reset parts
        if key == 'logins.username' or key == 'logins.password':
            config.set(key, DEFAULT_MSG)
        elif key == 'concurrency':
            config.set(key, DEFAULT_CONCURRENCY)
        elif key == 'dest_path':
            config.set(key, DEFAULT_DIR)
        else:
            print_error("Key does not exist!")
            return
    config.save()
    print_success("Config has been reset!")

def update_config(config, key, value):
    try:
        config.set(key, value)
        config.save()
    except Exception as e:
        print_error(e.message)
    else:
        print_success("Config updated and saved!")

def main():
    """
    Parse cli args and launch the program
    """
    init_terminal()
    cmd = sys.argv[1]
    config = ConfigParser()
    if cmd == 'dl':
        download(config, sys.argv[2])
    elif cmd == 'reset':
        reset_config(config, sys.argv[2])
    elif cmd == 'config':
        update_config(config, sys.argv[2], sys.argv[3])
    else:
        print_error("Command unrecognized!")

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from builtins import input
from builtins import object
import os
import sys
import json
import requests
from getpass import getpass
from .utils import *
from .constants import *

"""
Lecture authentication procedures

@Version 2.0.0
"""
class Authenticator(object):

    def __init__(self):
        """
        Inject username & password to the authenticator
        
        Arguments:
            username {str} -- Umich uniqname
            password {str} -- Uniqname password
        """
        self._service_types = ["cosign-ctools", "cosign-shibboleth.umich.edu", "cosign-leccap.engin"]
        self._session = requests.Session()
        self._username = None
        self._password = None
    
    def authenticate(self, service=None):
        """
        Authenticating various umich services
        Credits to Maxim The Man

        Keyword Arguments:
            service {str} -- Service Type (default: {None})
        """
        if not service:
            for st in self._service_types:
                self._authenticate_service(st)
        else:
            self._authenticate_service(service)
    
    def ask_for_credentials(self, username=None, password=None):
        """
        Retrieve the credentials
        
        Keyword Arguments:
            username {str} -- login username (default: {None})
            password {str} -- login password (default: {None})
        """
        if username and password:
            self._username = username
            self._password = password
        else:
            # ask for raw input
            self._username = input("Your uniqname: ")
            self._password = getpass("Your password: ")       

    def is_authenticated(self):
        """
        Whether the user has authenticated

        TODO: buggy if cookie expired, but usually fine
        """
        return bool(self._session.cookies.get("cosign-weblogin"))

    def session(self):
        """
        Get the underlining requests session
        """
        return self._session

    """
    Helpers
    """
    def _authenticate_service(self, service_type):
        # load login page to get cookie
        self._session.get(AUTH_PAGE_URL)
        # post to login
        self._session.post(AUTH_URL, {
            "service": service_type,
            "required": "",
            "login": self._username,
            "password": self._password
        })




from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from builtins import input
from builtins import object
import os
import sys
import wget
import json
import time
import threading
from queue import Queue
from .utils import *
from .constants import *

"""
Lecture download procedures

@Version 2.0.0
"""
class Downloader(object):
    
    class ProgressReporter(threading.Thread):
        """
        Report progress for multiple download threading
        
        Arguments:
            threading {Thread}
        """
        def __init__(self, num_workers, msg_queue):
            threading.Thread.__init__(self)
            self._num_workers = num_workers
            self._msg_queue = msg_queue
        
        def run(self):
            """
            Check and report progress
            """
            status = {}
            while self._num_workers > 0:
                msg = self._msg_queue.get()
                # check status
                if msg['type'] == "update":
                    status[msg['title']] = {
                        'current': msg['current'],
                        'total': msg['total']
                    }
                    self._show_progress(status)
                elif msg['type'] == "done":
                    self._num_workers -= 1
        
        def _show_progress(self, status):
            """
            Print the progress in console
            """
            current = 0
            total = 0
            for title, data in list(status.items()):
                current += data['current']
                total += data['total']
            line = "%s / %s bytes (%s recording(s)) in progress" % (current, total, len(status))
            self._print_message(print_info(line, True))
        
        def _print_message(self, msg):
            """
            Print message using stdout
            
            Arguments:
                msg {str}
            """
            sys.stdout.write("\r" + msg)
            sys.stdout.flush()
    
    class Task(threading.Thread):
        """
        Download Thread
        
        Arguments:
            threading {Thread}
        """
        def __init__(self, url, dest_path, filename, msg_queue):
            threading.Thread.__init__(self)
            self._url = url
            self._dest_path = dest_path
            self._filename = filename
            self._msg_queue = msg_queue

        def run(self):
            """
            Run the download task
            """
            filepath = os.path.join(self._dest_path, self._filename)
            # custom progress message
            custom_bar = self._send_updates()
            wget.download(self._url, out=filepath, bar=custom_bar)
            # completion message
            self._send_completion()

        def _send_updates(self):
            def custom_bar(current, total, width=None):
                self._msg_queue.put({
                    'type': 'update',
                    'title': self._filename,
                    'current': current,
                    'total': total
                })
                return None
            return custom_bar
        
        def _send_completion(self):
            self._msg_queue.put({
                'type': 'done',
                'title': self._filename,
                'current': 'Completed',
                'total': 'Completed'
            })

    def __init__(self, url, dest_path):
        """
        Initialize with download configuration
        
        Arguments:
            url {str} -- Target url, either a site or actual video
            concurrency {int} -- Number of download process to run
        """
        self._url = url
        self._dest_path = dest_path
        self._authenticator = None
    
    def set_concurrency(self, conc):
        """
        Inject concurrency
        
        Arguments:
            conc {int} -- Number of threads to run at same time
        """
        self._concurrency = conc

    def set_auth(self, auth):
        """
        Inject authenticator
        
        Arguments:
            auth {Authenticator}
        """
        self._authenticator = auth

    def get_auth(self):
        """
        Retrieve the authenticator
        
        Returns:
            Authenticator
        """
        return self._authenticator

    def requires_auth(self):
        """
        Whether this downloader requires authentication
        
        Returns:
            bool
        """
        return True

    def start(self):
        """
        Check argument and start downloading
        """
        # start downloading
        try:
            if self._is_recording_url(self._url):
                self._download_video()
            elif self._is_site_url(self._url):
                self._download_site()
            else:
                print_error("The URL you entered it not valid!")
        except Exception as e:
           print_error(e.message)
           print_warning("This is probably because of wrong credentials.")

    """
    Helpers
    """
    def _download_video(self):
        # extract video id
        video_id = self._url.split('/')[-1]
        # fetch data from api
        res = self._authenticator.session().get("%s?rk=%s" % (API_BASE_URL, video_id))
        recordings = json.loads(res.text)
        # find the specific recording
        for recording in recordings:
            if recording['recordingkey'] == video_id:
                self._download_with_thread([recording])
                break

    def _download_site(self):
        # extract site id
        site_id = self._url.split('/')[-1]
        # fetch data from api
        res = self._authenticator.session().get("%s?sk=%s" % (API_BASE_URL, site_id))
        recordings = json.loads(res.text)
        # show list of downloadable recordings
        print_info("%i recording%s available:" % (
            len(recordings), ("s" if (len(recordings) != 1) else "")))
        # print lecture info
        for idx, recording in enumerate(recordings):
            print("%i\t(%s)\t%s" % (
                idx + 1, recording["date"], recording["title"]))
        # prompt user to select
        prompt = print_info("Please select one or more comma separated, e.g 1,3,5. Or simply enter to download all of them: ", True)
        ans = input(prompt)
        if ans == '':
            to_dl = recordings
        else:
            to_dl = [ recordings[int(i)-1] for i in ans.split(',') ]
        # download the recordings
        self._download_with_thread(to_dl)
        
    def _download_with_thread(self, recordings):       
        # split recordings into batch of concurrencies
        batches = list(chunks(recordings, int(self._concurrency)))
        for batch_idx, batch in enumerate(batches):
            print_info("Start downloading batch %s ... " % (batch_idx + 1))
            # load a batch
            tasks = []
            msg_queue = Queue()
            for rec_idx, recording in enumerate(batch):
                # add task
                dl_url = self._construct_recording_file_url(recording)
                title = self._construct_recording_title(rec_idx, recording)
                dest_path = os.path.join(self._dest_path, recording['sitename'])
                # check directory
                if not os.path.exists(dest_path):
                    os.makedirs(dest_path)
                task = self.Task(dl_url, dest_path, title, msg_queue)
                tasks.append(task)
            # add the progress reporter
            reporter = self.ProgressReporter(len(tasks), msg_queue)
            tasks.append(reporter)
            # download with threads
            for t in tasks:
                t.start()
            # wait for all of them to finish
            for t in tasks:
                t.join()
            print('') # for new line
        print_success("All done! Enjoy!")

    def _is_recording_url(self, url):
        return "https://leccap.engin.umich.edu/leccap/viewer" in url

    def _is_site_url(self, url):
        return "https://leccap.engin.umich.edu/leccap/site" in url

    def _construct_recording_file_url(self, recording):
        return "https:%s%s/%s.%s" % (recording["mediaPrefix"],
                                     recording["sitekey"],
                                     recording["info"]["movie_exported_name"],
                                     recording["info"]["movie_type"])
    
    def _construct_recording_title(self, idx, recording):
        title = recording['title'].replace('[',"").replace(']',"").replace('/', "-")
        ext = recording['info']['movie_type']
        return "%s-%s.%s" % (idx, title, ext)



from __future__ import absolute_import
from .config import ConfigParser

def test_config():
    c = ConfigParser()
    c.set("logins.username", "test1")
    assert c.get("logins.username") == "test1"
    c.set("concurrency", 10)
    assert c.get("concurrency") == 10

def test_all():
    test_config()

if __name__ == "__main__":
    test_all()



from distutils.core import setup
# TEST Package
# setup(
#     name='leccap-test',
#     packages=['leccap'],
#     version='0.0.5',
#     description='Umich lecture downloader',
#     author='Jackie Zhang',
#     author_email='jackierw@umich.edu',
#     url='https://github.com/ReactiveXYZ-Dev/dleccap',
#     keywords=['leccap', 'downloader', 'lecture', 'umich'],
#     license="CC0",
#     classifiers=["Programming Language :: Python :: 2"],
#     scripts=['bin/leccap'],
#     package_data={
#         'leccap': ['*.json'],
#     },
#     install_requires=['requests', 'wget', 'colorama', 'future']
# )

setup(
    name='leccap',
    packages=['leccap'],
    version='0.0.5',
    description='Umich lecture downloader',
    author='Jackie Zhang',
    author_email='jackierw@umich.edu',
    url='https://github.com/ReactiveXYZ-Dev/dleccap',
    keywords=['leccap', 'downloader', 'lecture', 'umich'],
    license="CC0",
    scripts=['bin/leccap'],
    package_data={
        'leccap': ['*.json'],
    },
    install_requires=['requests', 'wget', 'colorama', 'future']
)
